#!/usr/bin/perl
#
# seekcp - seek a specified checkpoint in NILFS
#
# Copyright (C) 2006 Nippon Telegraph and Telephone Corporation.
# This file is part of NILFS and is licensed under the GPL.
#
# seekcp.pl,v 1.11 2007/02/07 04:58:03 moriai Exp
#
# Written by Satoshi Moriai <moriai@osrg.net>
#

use POSIX qw(mktime);

use constant {SIMPLE => 0, COMPACT => 1, LONG => 2, HTML => 3, DATE => 4};
my $mode = SIMPLE;
my $once = 1;

my %MM = (
    Jan => 1, Feb => 2, Mar => 3, Apr => 4, May => 5, Jun => 6,
    Jul => 7, Aug => 8, Sep => 9, Oct => 10, Nov => 11, Dec => 12
);

sub usage {
    my ($code) = @_;

    print <<"EOF";
Usage: seekcp [option] device [date]
Options:
  -h, --help			show this help
  -l, --long			use a long listing format
  -c, --compact			use a compact listing format
  -x, --html			use a HTML listing format

Acceptable date formats:
  YYYY.MM.DD[.hh[.mm[.ss]]]	e.g. 2006.03.02.12.34.56
  YYYY_Mmm_DD_[hh:mm[:ss]]	e.g. 2006_Mar_02_12:34:56
  Mmm DD [YYYY] [hh:mm[:ss]]	e.g. Mar 02 2006 12:34:56
  Mmm DD [hh:mm[:ss]] [YYYY]	e.g. Mar 02 12:34:56 2006
  DD Mmm [YYYY] [hh:mm[:ss]]	e.g. 02 Mar 2006 12:34:56
  YYYY-MM-DD [hh:mm[:ss]]	e.g. 2006-03-02 12:34:56
  YYYY/MM/DD [hh:mm[:ss]]	e.g. 2006/03/02 12:34:56
EOF

    exit $code;
}

sub maketime {
    my ($str) = @_;
    my ($Y, $M, $D, $h, $m, $s, $t);
    my ($G, $T);

    if ($str =~ /^\d{4}\./) {				# YYYY.MM.DD.hh.mm.ss
	($Y, $M, $D, $h, $m, $s) = split /\./, $str;
    } else {
	for $tk (split /[_ ]+/, $str) {
	    if ($tk =~ /^[A-Z][a-z]{2}$/) {			# Mmm
		$M = $MM{$tk};
	    } elsif ($tk =~ /^\d{4}-\d{1,2}-\d{1,2}$/) {	# YYYY-MM-DD
		($Y, $M, $D) = split /-/, $tk;
	    } elsif ($tk =~ /^\d{4}\/\d{1,2}\/\d{1,2}$/) {	# YYYY/MM/DD
		($Y, $M, $D) = split '/', $tk;
	    } elsif ($tk =~ /^\d{1,2}:\d{1,2}/) {		# hh:mm:ss
		($h, $m, $s) = split /:/, $tk;
	    } elsif ($tk =~ /^\d{4}$/) {			# YYYY
		$Y = $tk;
	    } elsif ($tk =~ /^\d{1,2}$/) {			# DD
		$D = $tk;
	    } else {
		usage(1);
	    }
	}
    }

    $Y = ((localtime)[5] + 1900) unless defined $Y;
    if (!defined $M && !defined $D) {
	($D, $M) = (localtime)[3,4];
	$M++;
    }
    $M = 12 unless defined $M;
#   $D = 31 unless defined $D;
    usage(1) unless defined $D;
    $h = 23 unless defined $h;
    $m = 59 unless defined $m;
    $s = 59 unless defined $s;
    $t = mktime($s, $m, $h, $D, $M - 1, $Y - 1900);
#   print STDERR "$Y-$M-$D $h:$m:$s   $t\n"; $t;	# DEBUG
}

sub printdata {
    printf "%04d.%02d.%02d.%02d.%02d.%02d %d %d %d\n", @_;
}

sub printhead {
    print <<EOF;
<html>
<head>
<script language="JavaScript">
<!--
var head;
function setHead() {
    var p = parent.location.href;
    head = p.slice(0, -14);
}
function getTail(s) {
    var n;
    if (!head) setHead();
    s = s.substring(head.length + 1);
    n = s.indexOf("/");
    if (n >= 0)
        return s.substring(n);
    return "";
}
function gotoRoot(s) {
    if (!head) setHead();
    parent.Content.location.href = head + s;
}
function changeRoot(s) {
    if (!head) setHead();
    try {
        parent.Content.location.href =
                head + s + getTail(parent.Content.location.href);
    } catch(e) {
        gotoRoot(s);
    }
}
// -->
</script>
</head>
<body>
<table border="0" cellpadding="0">
<col span="1" align="left"><col span="1" align="right">
EOF
}

sub printentry {
    my ($W, $Mmm, $M, $D, $h, $m, $s, $Y, $ssize) = @_;
    my ($name, $date);

    $name = sprintf("%04d.%02d.%02d.%02d.%02d.%02d", $Y, $M, $D, $h, $m, $s);
    $date = sprintf("%s %s %2d %02d:%02d:%02d %d", $W, $Mmm, $D, $h, $m, $s, $Y);
    $date =~ s/ /&nbsp;/g;
    $ssize *= 4;

    print <<EOF;
<tr>
<td><a href="javascript:void(0)" target="Content" onClick="gotoRoot('$name')" onMouseOver="changeRoot('$name')"><tt>$date</tt></a></td><td><tt>&nbsp;$ssize KB</tt></td>
</tr>
EOF
}

sub printtail {
    print <<EOF;
<tr>
<td><a href="javascript:void(0)" target="Content" onClick="gotoRoot('cur')" onMouseOver="changeRoot('cur')"><tt>Current</tt></a></td>
</tr>
</table>
</body>
</html>
EOF
}

#
# main routine
#

while ($_ = $ARGV[0], /^-/) {
    shift;
    if (/-h/ || /--help/) { usage(0); }
    if (/-t$/ || /--tail/) { $once = 0; next; }
    if (/-l$/ || /--long/) { $mode = LONG; last; }
    if (/-c$/ || /--compact/) { $mode = COMPACT; last; }
    if (/-x$/ || /--html/) { $mode = HTML; last; }
    if (/-d$/ || /--date/) { $mode = DATE; last; }
}

if ($#ARGV < 0) {
    usage(1);
}

if ($> != 0) {
    print STDERR "must be root!\n";
    exit 1;
}

open LISTCP, "printf 'listcp\nquit\n' | inspect $ARGV[0] |"
    || die "Can't popen listcp $!";

if ($#ARGV >= 1) {
    shift;
    $stime = maketime(join ' ', @ARGV);
#   printf STDERR "%d\n", $stime;			# DEBUG
}

if ($mode == HTML) {
    printhead();
}

while (<LISTCP>) {
    chop;
    s/nilfs> //;
    next unless /MajorCP/;

    ($str = $_) =~ s/^ +//;
    ($sno, $ssize, $W, $Mmm, $D, $T, $Y, $sstat, $sksize) = split / +/, $str;
    $M = $MM{$Mmm};
    ($h, $m, $s) = split /:/, $T;

    if (defined $stime) {
	last if ($matched);
	next if (mktime($s, $m, $h, $D, $M - 1, $Y - 1900) < $stime);
	$matched = 1 if ($once);
    }

    if ($mode == LONG) {
	print "$_\n";
    } elsif ($mode == COMPACT) {
	printdata($Y, $M, $D, $h, $m, $s, $sno, $ssize, $sksize, $sstat);
    } elsif ($mode == DATE) {
        printf "%04d.%02d.%02d.%02d.%02d.%02d\n", $Y, $M, $D, $h, $m, $s;
    } elsif ($mode == HTML) {
	printentry($W, $Mmm, $M, $D, $h, $m, $s, $Y, $ssize);
    } else {
	if (defined $delim) {
	    print $delim;
	} else {
	    $delim = ' ';
	}
	print $sno;
    }
}

if ($mode == SIMPLE && defined $delim) {
    print "\n";
}

if ($mode == HTML) {
    printtail();
}

if (defined $stime && !defined $matched && $once) {
    exit 1;
}

exit 0;
